import sys
import os
import traceback
import decimal
import datetime as dt
from datetime import datetime, timedelta
import numpy as np
from PyQt5.QtWidgets import *
from PyQt5 import uic
from PyQt5 import QtCore
from PyQt5.QtCore import *
# import sip
import pathlib
import matplotlib.pyplot as plt
from matplotlib.figure import Figure
from matplotlib.backends.backend_qt5agg import (FigureCanvasQTAgg as FigureCanvas,
                                                NavigationToolbar2QT as NavigationToolbar)
import csv
import rappmysql
from rappmysql.masina.auto import Masina, AutoApp
from rappmysql.masina.auto import DB_Connection as auto_db_connection
from rappmysql.mruser.myusers import Users
from rappmysql.mruser.myusers import DB_Connection as users_db_connection

np.set_printoptions(linewidth=600)

compName = os.getenv('COMPUTERNAME')


table_head_conversion = {
    'table_last_records': ['data', 'type', 'brutto', 'amount', 'eProvider', 'km'],
    'TW_all_alimentari': ['data', 'type', 'brutto', 'amount', 'eProvider', 'km'],
}
path2GUI = pathlib.Path(__file__)
path2GUI = path2GUI.resolve(path2GUI).parent / 'GUI'


class MyApp(QMainWindow):
    def __init__(self, users_db, auto_db):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        super(MyApp, self).__init__()
        main_window = path2GUI / 'masina.ui'
        Ui_MainWindow, QtBaseClass = uic.loadUiType(main_window)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        # self.conf = connect.Config(rappmysql.ini_masina)
        # self.setWindowTitle('{}'.format(self.conf.credentials['database']))
        self.users_db_connection = users_db
        self.auto_db_connection = auto_db

        fig = Figure()
        self.add_plots(fig)

        self.ui.GB_add_alim.setVisible(False)
        self.ui.GB_user_options.setVisible(False)
        self.ui.TW_main_user_interface.setVisible(False)
        self.ui.CB_SQL_table_head.setVisible(False)
        self.ui.PB_erase_traces.setVisible(False)

        self.ui.PB_login.clicked.connect(self.login_user)
        self.ui.PB_add_car.clicked.connect(self.add_car)
        self.ui.PB_fire_add_row.clicked.connect(self.add_row)
        self.ui.PB_fire_filter.clicked.connect(self.populateDetailsTab)
        self.ui.TB_add_file.clicked.connect(self.get_file_pth)
        self.ui.PB_Export_csv.clicked.connect(self.export_CSV)
        self.ui.PB_export_car_sql.clicked.connect(self.export_car_sql)
        self.ui.PB_export_profile.clicked.connect(self.export_full_profile)
        self.ui.PB_import_profile.clicked.connect(self.import_profile)
        self.ui.PB_erase_traces.clicked.connect(self.erase_traces)
        self.ui.PB_calc_fuel_consumption_since_last_refuel.clicked.connect(self.calc_fuel_consumption_since_last_refuel)
        self.ui.TW_all_alimentari.horizontalHeader().sectionClicked.connect(self.sortPlan)
        self.ui.TW_all_alimentari.itemDoubleClicked.connect(self.upload_download_file)
        self.ui.CB_alim_types.currentIndexChanged.connect(self.populate_GB_additional_for_add_row)
        self.ui.CBMonths.currentIndexChanged.connect(self.populateDatesInterval)
        self.ui.CB_add_alim.toggled.connect(self.add_cost)
        self.ui.CB_add_costs_additional_info.toggled.connect(self.add_costs_additional_info)
        self.ui.TW_plot_summary_det.currentChanged.connect(self.TW_plot_summary_det_currentChanged)
        self.ui.LE_amount.editingFinished.connect(self.calculate_electric_cost)

        if compName == 'DESKTOP-5HHINGF' or compName == 'MPCC6995':
            self.ui.LE_user_name.setText('radu')
            self.ui.LE_user_pass.setText('9876')
            self.login_user()

    def calc_fuel_consumption_since_last_refuel(self):
        actual_no_of_km = int(self.ui.LE_actual_no_of_km.text())
        benzin_left_in_tank = int(self.ui.LE_benzin_left_in_tank.text())
        # average_kwh_used = float(self.ui.LE_kwh_used.text())

        # consumption_since_refueling = self.app.since_refueling_consumption(actual_no_of_km, benzin_left_in_tank,
        #                                                                    average_kwh_used)
        consumption_since_refueling = self.app.calc_long_term_consumption(actual_no_of_km, benzin_left_in_tank)

        self.ui.LE_since_refuel_start_datum.setText(str(consumption_since_refueling['date_when_i_bought_the_car']))
        self.ui.LE_since_refuel_date_now.setText(str(consumption_since_refueling['today']))
        self.ui.LE_since_refuel_km_since_last_refuel.setText(str(consumption_since_refueling['km_driven_bis_datum']))
        self.ui.LE_since_refuel_liters_used.setText(str(consumption_since_refueling['liters_used']))
        self.ui.LE_since_refuel_fuel_consumption_since_last_refuel.setText(
            str(consumption_since_refueling['average_fuel_consumption']))

    def TW_plot_summary_det_currentChanged(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        tab_name = self.ui.TW_plot_summary_det.tabText(self.ui.TW_plot_summary_det.currentIndex())
        if tab_name == 'Plots':
            self.prep_2Dplot()

    def login_user(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        user_name = self.ui.LE_user_name.text()
        password = self.ui.LE_user_pass.text()
        self.user = Users(user_name=user_name,
                          users_table=self.users_db_connection.users_table,
                          user_apps_table=self.users_db_connection.user_apps_table)

        if self.user.verify_password(password):
            self.user.auto_app = AutoApp(auto_db=self.auto_db_connection.auto_db,
                                         all_cars_table=self.auto_db_connection.all_cars_table,
                                         alimentari=self.auto_db_connection.alimentari,
                                         user_id=self.user.id)

            self.ui.GB_user_options.setVisible(True)
            if self.user.admin:
                self.ui.CB_SQL_table_head.setVisible(True)
                self.ui.PB_erase_traces.setVisible(True)

            self.ui.TW_main_user_interface.setVisible(True)
            all_user_cars = list(self.user.auto_app.masini.values())
            if all_user_cars:
                self.ui.CB_all_user_cars.addItems(all_user_cars)
                self.ui.CB_all_user_cars.setCurrentText(all_user_cars[-1])
                self.ui.TW_main_user_interface.setTabText(0, all_user_cars[-1])
                table_name = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
                id_car = int(self.user.auto_app.get_id_all_cars(table_name))
                self.app = Masina(auto_db=self.auto_db_connection.auto_db,
                                  all_cars_table=self.auto_db_connection.all_cars_table,
                                  alimentari=self.auto_db_connection.alimentari,
                                  id_users=self.user.id,
                                  id_car=id_car)
                self.prepare_masina_interface()
                self.prep_2Dplot()
                self.ui.CB_all_user_cars.currentIndexChanged.connect(lambda: (self.fill_in_tab(), self.prep_2Dplot()))
            else:
                self.ui.TW_main_user_interface.setVisible(False)
        self.ui.GB_login.setVisible(False)
        self.ui.GB_user_options.setTitle('Hello {}'.format(user_name))

    def erase_traces(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.user.auto_app.erase_autoapp_traces()
        print('Done')

    def calculate_electric_cost(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        amount = self.ui.LE_amount.text()
        if ',' in amount:
            amount = amount.replace(',', '.')
        amount = float(amount)
        eProvider = self.ui.CB_eProvider.currentText()
        ppu = self.user.auto_app.get_ppu_electric_provider(eProvider)
        brutto = amount * ppu
        self.ui.LE_brutto.setText(str(brutto))
        self.ui.label_brutto.setVisible(True)
        self.ui.LE_brutto.setVisible(True)
        self.ui.label_19.setVisible(True)

    def add_cost(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if self.ui.CB_add_alim.isChecked():
            self.ui.GB_add_alim.setVisible(True)
        else:
            self.ui.GB_add_alim.setVisible(False)

    def add_car(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        answer = AddCarWindow.get_def_arguments(self.user.auto_app.all_cars_array)
        if not answer:
            return
        else:
            car_type, car_brand, car_model = answer
        self.user.auto_app.add_car(car_brand, car_model, car_type)

    def prepare_masina_interface(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.populateSummaryTab()

        self.populate_CB_alim_types()
        self.populateCBMonths()
        self.alim_date()
        self.populateDatesInterval()
        self.populateDetailsTab()

    def fill_in_tab(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        current_car = self.ui.CB_all_user_cars.currentText()
        self.ui.TW_main_user_interface.setTabText(0, current_car)
        table_name = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        id_car = int(self.user.auto_app.get_id_all_cars(table_name))
        self.app = Masina(auto_db=self.auto_db_connection.auto_db,
                          all_cars_table=self.auto_db_connection.all_cars_table,
                          alimentari=self.auto_db_connection.alimentari,
                          id_users=self.user.id,
                          id_car=id_car)
        self.prepare_masina_interface()

    def prepare_login_window(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.ui.TW_main_user_interface.setVisible(False)
        self.ui.PB_export_profile.setVisible(False)
        if not self.app_users.all_users:
            self.ui.PB_login.setVisible(False)

    def populate_table_widget(self, widget, table):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        tableHead, table_data = list(table[0]), table[1:]
        if self.ui.CB_SQL_table_head.isChecked() or widget.objectName() not in list(table_head_conversion.keys()):
            widget.setColumnCount(len(tableHead))
            widget.setHorizontalHeaderLabels(tableHead)
            widget.setRowCount(table_data.shape[0])
            for col in range(table_data.shape[1]):
                for row in range(table_data.shape[0]):
                    item = QTableWidgetItem(str(table_data[row, col]))
                    widget.setItem(row, col, item)
        else:
            newTableHead = table_head_conversion[widget.objectName()]
            widget.setColumnCount(len(newTableHead))
            widget.setHorizontalHeaderLabels(newTableHead)
            widget.setRowCount(table_data.shape[0])
            for col in range(table_data.shape[1]):
                # print(tableHead[col], newTableHead, tableHead[col] in newTableHead)
                if tableHead[col] in newTableHead:
                    for row in range(table_data.shape[0]):
                        # if isinstance(table_data[row, col], int) or isinstance(table_data[row, col], float):
                        #     item = QTableWidgetItem()
                        #     item.setData(QtCore.Qt.DisplayRole, table[row, col])
                        # elif isinstance(table_data[row, col], decimal.Decimal):
                        #     val = float(table_data[row, col])
                        #     item = QTableWidgetItem()
                        #     item.setData(QtCore.Qt.DisplayRole, val)
                        # # elif tableHead[col] == 'file_name':
                        # #     text = "<a href={}>{}</a>".format(str(table_data[row, col]), str(table_data[row, col]))
                        # #     item = QTableWidgetItem(text)
                        # #     # item.setText(text)
                        # #     # item.setOpenExternalLinks(True)
                        # #     # item.clicked.connect(self.download_file)
                        # #     # widget.setCellWidget(row, col, item)
                        # #     # continue
                        # else:
                        #     item = QTableWidgetItem(str(table_data[row, col]))
                        new_col = newTableHead.index(tableHead[col])
                        item = QTableWidgetItem(str(table_data[row, col]))
                        widget.setItem(row, new_col, item)
        header = widget.horizontalHeader()
        header.setSectionResizeMode(QHeaderView.ResizeToContents)

    def upload_download_file(self, item):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print(item.column(), item.row(), item.text())
        if self.ui.TW_all_alimentari.horizontalHeaderItem(item.column()).text() == 'file_name':
            # print(item)
            id_ = self.ui.TW_all_alimentari.item(item.row(), 0).text()
            if item.text() != 'None':
                # print('ÄÄ', id_)
                print('id', id_)
                print('ÖÖ', self.ui.TW_all_alimentari.horizontalHeaderItem(0).text())
                expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "")
                print(expName)
                if expName:
                    file_content = self.app.alimentari.returnCellsWhere('file', ('id', id_))[0]
                    self.app.alimentari.write_file(file_content, expName)
                else:
                    return
            else:
                print('upload')
                uploadFile, _ = QFileDialog.getOpenFileName(self, "upload file", "", "")
                self.app.upload_file(uploadFile, id_)
                self.populateDetailsTab()

    def populateSummaryTab(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print('***self.app.table_totals', self.app.table_totals)
        if self.app.table_totals is not None:
            self.populate_table_widget(self.ui.table_totals, self.app.table_totals)
            self.populate_table_widget(self.ui.table_alimentari, self.app.table_alimentari)
            self.populate_table_widget(self.ui.table_last_records, self.app.last_records)
            self.populate_table_widget(self.ui.TW_vw_ab_tanken, self.app.vw_ab_tanken)

        self.ui.LE_long_term_start_date.setText(str(self.app.long_term_consumption['start']))
        self.ui.LE_long_term_end_date.setText(str(self.app.long_term_consumption['end']))
        self.ui.LE_km_driven_between_first_and_last_refueling.setText(str(self.app.long_term_consumption['km']))
        self.ui.LE_liters_refueled_between_first_and_last_refueling.setText(str(self.app.long_term_consumption['liters']))
        self.ui.LE_average_fuel_consumption.setText(str(self.app.long_term_consumption['average']))

        self.ui.LE_date_at_first_recharge.setText(str(self.app.long_term_consumption['date_at_first_recharge']))
        self.ui.LE_date_at_last_recharge.setText(str(self.app.long_term_consumption['date_at_last_recharge']))
        self.ui.LE_km_driven_between_first_and_last_recharge.setText(str(self.app.long_term_consumption['km_driven_between_first_and_last_recharge']))
        self.ui.LE_kWh_recharged_between_first_and_last_recharge.setText(str(self.app.long_term_consumption['kWh_recharged_between_first_and_last_recharge']))
        self.ui.LE_average_electric_consumption.setText(str(self.app.long_term_consumption['average_electric_consumption']))

    def populateDetailsTab(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # self.populate_table_widget(self.ui.TW_all_alimentari, self.app.get_all_alimentari())
        selectedStartDate = self.ui.DEFrom.date().toPyDate()
        selectedEndDate = self.ui.DEBis.date().toPyDate()
        alim_type = self.ui.CB_alim_types_filter.currentText()
        if self.ui.CB_alim_types_filter.currentText() == 'all':
            alim_type = None
        res_table = self.app.get_alimentari_for_interval_type(selectedStartDate, selectedEndDate, alim_type)
        self.populate_table_widget(self.ui.TW_all_alimentari, res_table)
        self.ui.LE_filter_showing.setText(str(res_table.shape[0]))
        self.ui.LE_total_rows.setText(str(self.app.alimentari.noOfRows))
        return res_table

    def add_row(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print('self.app.insert_new_alim()')
        data = self.ui.alim_date.date().toPyDate()
        alim_type = self.ui.CB_alim_types.currentText()
        brutto = self.ui.LE_brutto.text()
        file = self.ui.LE_add_file.text()

        amount = self.ui.LE_amount.text()
        refuel = self.ui.LE_refuel.text()
        other = self.ui.LE_other.text()
        recharges = self.ui.LE_recharges.text()
        provider = self.ui.CB_eProvider.currentText()
        km = self.ui.LE_km.text()
        comment = self.ui.LE_comment.text()
        table_name = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        id_all_cars = self.user.auto_app.get_id_all_cars(table_name.lower())
        # print('**', self.user.id, table_name.lower(), id_all_cars)
        # current_user_id, id_all_cars, data, alim_type, brutto, amount, refuel, other, recharges, km, comment, file
        self.app.insert_new_alim(current_id_users=self.user.id,
                                 id_all_cars=id_all_cars,
                                 data=data,
                                 alim_type=alim_type,
                                 brutto=brutto,
                                 file=file,
                                 amount=amount,
                                 refuel=refuel,
                                 other=other,
                                 recharges=recharges,
                                 provider=provider,
                                 km=km,
                                 comment=comment)

    def populate_CB_alim_types(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.ui.CB_alim_types.addItems(self.app.types_of_costs)
        self.ui.CB_alim_types_filter.addItem('all')
        self.ui.CB_alim_types_filter.addItems(self.app.types_of_costs)

    def populate_GB_additional_for_add_row(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.ui.label_brutto.setVisible(False)
        self.ui.LE_brutto.setVisible(False)
        self.ui.label_19.setVisible(False)

        self.ui.label_amount.setVisible(False)
        self.ui.LE_amount.setVisible(False)
        self.ui.label_L_KwH.setVisible(False)

        self.ui.label_Provider.setVisible(False)
        self.ui.CB_eProvider.setVisible(False)

        self.ui.label_Recharges.setVisible(False)
        self.ui.LE_recharges.setVisible(False)

        self.ui.label_km.setVisible(False)
        self.ui.LE_km.setVisible(False)

        self.ui.GB_additional_info_by_add_costs.setVisible(False)

        if self.ui.CB_alim_types.currentText() == 'electric':
            self.ui.label_Provider.setVisible(True)
            self.ui.CB_eProvider.setVisible(True)

            # self.ui.label_brutto.setVisible(True)
            # self.ui.LE_brutto.setVisible(True)
            # self.ui.label_19.setVisible(True)
            #
            self.ui.label_amount.setVisible(True)
            self.ui.LE_amount.setVisible(True)
            self.ui.label_L_KwH.setVisible(True)
            self.ui.label_L_KwH.setText('KwH')

            self.ui.label_km.setVisible(True)
            self.ui.LE_km.setVisible(True)

            self.ui.CB_eProvider.addItem('')
            self.ui.CB_eProvider.addItems(self.user.auto_app.electric_providers)
        elif self.ui.CB_alim_types.currentText() == 'benzina':
            self.ui.label_brutto.setVisible(True)
            self.ui.LE_brutto.setVisible(True)
            self.ui.label_19.setVisible(True)

            self.ui.label_amount.setVisible(True)
            self.ui.LE_amount.setVisible(True)
            self.ui.label_L_KwH.setVisible(True)

            self.ui.label_km.setVisible(True)
            self.ui.LE_km.setVisible(True)

            # self.ui.GB_Refuel.setVisible(True)
            # # self.ui.horizontalLayout_6.setVisible(False)
            # self.ui.GB_other.setVisible(True)
            # self.ui.GB_Charges.setVisible(True)
            # self.ui.GB_Km.setVisible(True)
            # self.ui.GB_Comment.setVisible(True)
            # self.ui.label_L_KwH.setText('L')
            # self.ui.label_Refuel_eCharge.setText('Refuel')
        elif self.ui.CB_alim_types.currentText() == 'intretinere':
            self.ui.GB_Km.setVisible(True)
            self.ui.GB_Comment.setVisible(True)
        else:
            self.ui.GB_Comment.setVisible(True)

    def alim_date(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.ui.alim_date.setDate(datetime.now())
        self.ui.alim_date.setCalendarPopup(True)

    def get_file_pth(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.loadFile, _ = QFileDialog.getOpenFileName(self, "File", "", "File (*.jpg;*.JPG;*.pdf)")
        if self.loadFile:
            self.ui.LE_add_file.setText(self.loadFile)

    def populateCBMonths(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        self.ui.CBMonths.addItem('interval')
        months = [dt.date(2000, m, 1).strftime('%B') for m in range(1, 13)]
        for month in months:
            self.ui.CBMonths.addItem(month)

        self.ui.SB_year.setValue(datetime.now().year)

    def populateDatesInterval(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        startDate, lastDayOfMonth = self.app.default_interval

        if self.ui.CBMonths.currentText() != 'interval':
            year = int(self.ui.SB_year.value())
            startDate, lastDayOfMonth = self.app.get_monthly_interval(self.ui.CBMonths.currentText(), year)

        self.ui.DEFrom.setDate(startDate)
        self.ui.DEBis.setDate(lastDayOfMonth)

        self.ui.DEFrom.setCalendarPopup(True)
        self.ui.DEBis.setCalendarPopup(True)

    def sortPlan(self, logical_index):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        header = self.ui.TW_all_alimentari.horizontalHeader()
        order = Qt.DescendingOrder
        if not header.isSortIndicatorShown():
            header.setSortIndicatorShown(True)
        elif header.sortIndicatorSection() == logical_index:
            order = header.sortIndicatorOrder()
        header.setSortIndicator(logical_index, order)
        self.ui.TW_all_alimentari.sortItems(logical_index, order)

    def export_CSV(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        expName, _ = QFileDialog.getSaveFileName(self, "Save file", "", "File (*.csv)")
        # print(expName)
        if expName:
            res_table = self.populateDetailsTab()
            with open(expName, 'w', newline='') as file:
                writer = csv.writer(file, delimiter=';')
                # if isinstance(array, list) or isinstance(array, np.ndarray):
                for row in res_table:
                    writer.writerow(row)

    def export_car_sql(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        table_name = self.ui.TW_main_user_interface.tabText(self.ui.TW_main_user_interface.currentIndex())
        id_car = int(self.user.auto_app.get_id_all_cars(table_name))
        output_sql_file = self.user.auto_app.export_car_sql(id_car)
        print('created: ', output_sql_file)
        return output_sql_file

    def export_full_profile(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        export_profile_with_files = False
        if self.ui.CB_export_profile_with_files.isChecked():
            export_profile_with_files = True

        for car_name in list(self.user.auto_app.masini.values()):
            print(car_name)
            id_car = int(self.user.auto_app.get_id_all_cars(car_name))
            output_sql_file = self.user.auto_app.export_car_sql(id_car, export_files=export_profile_with_files)
            print(output_sql_file)
        return

    def add_costs_additional_info(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        if self.ui.CB_add_costs_additional_info.isChecked():
            self.ui.GB_additional_info_by_add_costs.setVisible(True)
        else:
            self.ui.GB_additional_info_by_add_costs.setVisible(False)

    def import_profile(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        import_files = self.ui.CB_import_profile_with_files.isChecked()
        zipFile, _ = QFileDialog.getOpenFileName(self, "Open file", "", "File (*.zip)")
        if zipFile:
            self.user.auto_app.import_car_with_files(zipFile, import_files=import_files)

    def prep_2Dplot(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        fig1 = Figure()
        spec5 = fig1.add_gridspec(ncols=1, nrows=1)  # , height_ratios=[1, 1]
        ax1f2 = fig1.add_subplot(spec5[0, 0])
        ax1f2.grid(True)

        ########## bar_label  ########
        payments_dict = self.app.dict_last_months()
        labels = list(payments_dict.keys())
        x = np.arange(len(labels))  # the label locations
        sex_counts = {}
        for type_of_cost in self.app.types_of_costs:
            sex_counts[type_of_cost] = []
            for lab, vals in payments_dict.items():
                if type_of_cost not in vals.keys():
                    sex_counts[type_of_cost].append(0)
                else:
                    sex_counts[type_of_cost].append(vals[type_of_cost])

        bottom = np.zeros(len(labels))
        width = 0.6  # the width of the bars: can also be len(x) sequence
        for sex, sex_count in sex_counts.items():
            # print(' ** ** * labels', labels)
            # print(' ** ** * sex_count', sex_count, type(sex_count))
            # print(' ** ** * sex', sex)
            # print(labels, sex_count, width, sex, bottom)
            if list(set(sex_count)) == [0]:
                # print('ÖÖÖÖÖÖÖÖ', list(set(sex_count)))
                continue
            # print('ÖÖÖÖÖ', labels, sex_count, width, sex, bottom)

            p = ax1f2.bar(labels, sex_count, width, label=sex, bottom=bottom)
            bottom += sex_count
            # print(bottom)
            ax1f2.bar_label(p, label_type='center')
            # print(50*'K')

        ########## axvline  ########
        plot_lr = self.app.last_records_for_plot()
        cmap = plt.get_cmap('tab10')
        for idx, (k, v) in enumerate(plot_lr.items()):
            # print('ÖÖÖÖÖÖÖÖ', k)
            # print('ÄÄÄÄÄ', v)
            pos = v * len(labels)
            # color = colors[idx % len(colors)]  # Cycle through colors
            color = cmap(idx % cmap.N)
            ax1f2.axvline(x=pos, linewidth=2, label=k, color=color)  # color='#d62728'
        ax1f2.set_xticks(x)
        ax1f2.set_xticklabels(labels)
        # print('self.app.monthly_electric')
        # print(self.app.monthly_electric)
        # print()
        # print('self.app.monthly_benzina')
        # print(self.app.monthly_benzina)
        # print()
        # print('self.app.monthly')
        # print(self.app.monthly)
        # print()
        yticks = {'monthly_electric_{}'.format(self.app.monthly_electric): self.app.monthly_electric,
                  'monthly_benzina_{}'.format(self.app.monthly_benzina): self.app.monthly_benzina,
                  'monthly_{}'.format(self.app.monthly): self.app.monthly}

        yticks_vals = [x for x in yticks.values() if x is not None]
        yticklabels = [k for k, v in yticks.items() if v is not None]
        #     if tick is None:
        #         print('BINGO')
        ax1f2.set_yticks(yticks_vals, [])
        ax1f2.set_yticklabels(yticklabels)
        ax1f2.tick_params(axis='x', rotation=55)
        ax1f2.legend()
        self.rmmpl()
        self.add_plots(fig1)

    def add_plots(self, fig):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print(sys._getframe().f_code.co_name)
        self.canvas = FigureCanvas(fig)
        self.ui.loc_de_plot.addWidget(self.canvas)

    def rmmpl(self):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        # print(sys._getframe().f_code.co_name)
        self.ui.loc_de_plot.removeWidget(self.canvas)
        self.canvas.close()


class AddCarWindow(QDialog):
    def __init__(self, all_cars_array):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        super(AddCarWindow, self).__init__()
        add_auto_window = path2GUI / 'add_auto.ui'
        Ui_MainWindow, QtBaseClass = uic.loadUiType(add_auto_window)
        self.gui = Ui_MainWindow()
        self.gui.setupUi(self)

        self.all_cars_array = all_cars_array[1:]
        self.all_cars_array_table_head = list(all_cars_array[0])
        self.populate_combo_boxes()

        self.gui.CB_car_brand.currentIndexChanged.connect(self.combo_box_index_changed)
        self.gui.CB_car_model.currentIndexChanged.connect(self.combo_box_index_changed)

    def populate_combo_boxes(self):
        self.gui.CB_car_brand.addItem('')
        all_cars_brands = self.all_cars_array[:,self.all_cars_array_table_head.index('car brand')]
        self.gui.CB_car_brand.addItems(np.unique(all_cars_brands))
        self.gui.CB_car_model.addItem('')
        self.gui.CB_car_motorisation.addItem('')

    def combo_box_index_changed(self):
        if self.sender().objectName() == 'CB_car_brand':
            self.gui.CB_car_model.clear()
            self.gui.CB_car_motorisation.clear()
            car_brand = self.gui.CB_car_brand.currentText()
            brand_models = self.all_cars_array[self.all_cars_array[:,self.all_cars_array_table_head.index('car brand')] == car_brand]
            brand_models = np.unique(brand_models[:,self.all_cars_array_table_head.index('model')])
            self.gui.CB_car_model.addItems(brand_models)
        elif self.sender().objectName() == 'CB_car_model':
            car_brand = self.gui.CB_car_brand.currentText()
            self.gui.CB_car_motorisation.clear()
            car_model = self.gui.CB_car_model.currentText()
            if car_model != '':
                indx = np.where((car_brand == self.all_cars_array[:,self.all_cars_array_table_head.index('car brand')]) &
                                (car_model == self.all_cars_array[:,self.all_cars_array_table_head.index('model')]))
                models_motorisations = np.unique(self.all_cars_array[indx, self.all_cars_array_table_head.index('motorisation')][0])
                self.gui.CB_car_motorisation.addItems(models_motorisations)

    def load_inputs(self):
        car_brand = self.gui.CB_car_brand.currentText()
        car_type = self.gui.CB_car_type.currentText()
        car_model = self.gui.CB_car_motorisation.currentText()
        return car_type, car_brand, car_model

    @staticmethod
    def get_def_arguments(all_cars_array):
        print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name,
                                                                  sys._getframe().f_back.f_code.co_name))
        dialog = AddCarWindow(all_cars_array)
        result = dialog.exec_()
        if result == QDialog.Accepted:
            car_type, car_brand, car_model = dialog.load_inputs()
            return car_type, car_brand, car_model
        else:
            return None


def main():
    auto_db = auto_db_connection(rappmysql.ini_masina)
    users_db = users_db_connection(rappmysql.ini_users)

    app = QApplication(sys.argv)
    window = MyApp(users_db, auto_db)
    window.show()
    # sys.exit(app.exec_())
    app.exec_()


if __name__ == '__main__':
    main()



from datetime import datetime, date
from flask import Flask, render_template, request, redirect, url_for, session
from flask_login import UserMixin, login_user, LoginManager, login_required, logout_user, current_user
import traceback
from werkzeug.security import generate_password_hash
from werkzeug.utils import secure_filename
import re
import os
import sys
import csv
from rappmysql.cheltuieli import chelt_plan
from rappmysql.cheltuieli.chelt_plan import CheltuieliPlanificate
from rappmysql.mruser.myusers import Users
from rappmysql.masina.auto import Masina, AutoApp
from rappmysql.aeroclub.aeroclub import AeroclubSQL
from rappmysql.mruser.myusers import DB_Connection as users_db_connection
from rappmysql.cheltuieli.chelt_plan import DB_Connection as chelt_db_connection
from rappmysql.masina.auto import DB_Connection as auto_db_connection
import rappmysql

users_db = users_db_connection(rappmysql.ini_users)
auto_db = auto_db_connection(rappmysql.ini_masina)
chelt_db = chelt_db_connection(rappmysql.ini_chelt)

app = Flask(__name__)
app.config["DEBUG"] = False
app.config['SECRET_KEY'] = "my super secret"
login_manager = LoginManager()
login_manager.init_app(app)
report = '/home/radum/mysite/static/report.csv'


@login_manager.user_loader
def load_user(user_id):
    print('Module: {}, Def: {}, Caller: {}'.format(__name__,
                                                   sys._getframe().f_code.co_name,
                                                   sys._getframe().f_back.f_code.co_name))
    try:
        try:
            mat = int(user_id)
        except:
            mat = int(re.findall("\[(.+?)\]", user_id)[0])
        matches = ('id', mat)

        name = users_db.users_table.returnCellsWhere('username', matches)[0]
        user = Users(user_name=name,
                     users_table=users_db.users_table,
                     user_apps_table=users_db.user_apps_table)
        return user
    except:
        print(30 * '*', str(traceback.format_exc()))


@app.route('/', methods=['GET', 'POST'])
def index():
    try:
        return render_template('index.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route("/login", methods=["GET", "POST"])
def login():
    try:
        if request.method == "POST":
            username = request.form['username']
            user = Users(user_name=username,
                         users_table=users_db.users_table,
                         user_apps_table=users_db.user_apps_table)

            if user.verify_password(request.form['password']):
                login_user(user)
                return redirect(url_for("index"))
            else:
                return 'NOT LOGED IN'
        else:
            return render_template("login.html")
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/logout', methods=['GET', 'POST'])
@login_required
def logout():
    try:
        logout_user()
        return redirect(url_for('index'))
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/register', methods=['GET', 'POST'])
def register():
    try:
        if request.method == 'POST':
            username = request.form['username']
            email = request.form['email']
            password = request.form['password']
            cols = ('username', 'email', 'password')
            hash = generate_password_hash(password)
            vals = (username, email, hash)
            users_db.users_table.addNewRow(cols, vals)
        return render_template('register.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


###masina###################################################################################
@app.route('/masina', methods=['GET', 'POST'])
@login_required
def masina():
    print('Module: {}, Def: {}, Caller: {}'.format(__name__,
                                                   sys._getframe().f_code.co_name,
                                                   sys._getframe().f_back.f_code.co_name))

    try:
        print('ÖÖÖÖÖÖÖcurrent_user.auto_app', current_user.auto_app)
        if current_user.auto_app is None:
            current_user.auto_app = AutoApp(
                auto_db=auto_db.auto_db,
                all_cars_table=auto_db.all_cars_table,
                alimentari=auto_db.alimentari,
                user_id=current_user.id)
        # return '{}___{}'.format(current_user.user_name, str(current_user.auto_app.masini.items()))
        return render_template('masina.html')
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name,
                                           str(traceback.format_exc()))
        return render_template('err.html', myerr=myerr)


@app.route('/summary/<id_car>/<table_name>', methods=['GET', 'POST'])
@login_required
def car_summary(id_car, table_name):
    try:
        session['id_car'] = id_car
        if request.method == 'GET':
            # app_masina = Masina(ini_masina, current_user.id, id_car)
            app_masina = Masina(auto_db=auto_db.auto_db,
                                all_cars_table=auto_db.all_cars_table,
                                alimentari=auto_db.alimentari,
                                id_users=current_user.id,
                                id_car=id_car)
            return render_template('car_summary.html',
                                   table_name=table_name,
                                   tabel_alimentari=app_masina.table_alimentari,
                                   tabel_totals=app_masina.table_totals,
                                   last_records=app_masina.last_records
                                   )
        elif request.method == 'POST':
            if 'export_detail_table_as_sql' in request.form:
                output_sql_file = current_user.export_car_sql(id_car)
                # # return text_to_write
                # pth_to_sql = os.path.join(report_dir, 'aaa.sql')
                # # if os.path.exists(pth_to_sql):
                # #     os.remove(pth_to_sql)
                # # with open(pth_to_sql, 'w') as file:
                # #     for row in text_to_write:
                # #         file.write(row)
                return output_sql_file
            elif 'export_detail_table_with_files' in request.form:
                output_sql_file = current_user.export_car_sql(id_car, export_files=True)
                return output_sql_file

                # app_masina = Masina(conf.credentials, table_name=table_name.lower())
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html', myerr=myerr)


@app.route('/addalim/<table_name>', methods=['GET', 'POST'])
def addalim(table_name):
    try:
        if current_user.auto_app is None:
            current_user.auto_app = AutoApp(
                auto_db=auto_db.auto_db,
                all_cars_table=auto_db.all_cars_table,
                alimentari=auto_db.alimentari,
                user_id=current_user.id)

        app_masina = Masina(auto_db=auto_db.auto_db,
                            all_cars_table=auto_db.all_cars_table,
                            alimentari=auto_db.alimentari,
                            id_users=current_user.id,
                            id_car=session['id_car'])
        if request.method == 'GET':
            return render_template('addalim.html',
                                   table_name=table_name,
                                   types_of_costs=app_masina.types_of_costs)
        if request.method == 'POST':
            if "add_alim" in request.form:
                date = request.form['data']
                alim_type = request.form['type']
                eprov = request.form['eprov']
                brutto = request.form['brutto']
                amount = request.form['amount']
                km = request.form['km']
                # ppu = round(float(brutto)/float(amount), 3)
                # columns = ['data', 'type', 'eProvider', 'brutto', 'amount', 'ppu', 'km']
                # values = [date, alim_type, eprov, brutto, amount, ppu, km]

                # app_masina.insert_new_alim(columns, values)
                # id_car = current_user.auto_app.get_id_all_cars(table_name)
                app_masina.insert_new_alim(current_id_users=current_user.id, id_all_cars=session['id_car'], data=date,
                                           alim_type=alim_type, brutto=brutto, amount=amount, refuel=None, other=None,
                                           recharges=None, km=km, comment=None, file=None, provider=eprov)
                return redirect(url_for('car_summary', id_car=session['id_car'], table_name=table_name))
    except:
        myerr = 'a_{}_a definition {}\n{}'.format(current_user.id, sys._getframe().f_code.co_name,
                                                  str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/deleterow/<table_name>/<index>', methods=['GET', 'POST'])
def deleterow(table_name, index):
    try:
        app_masina = Masina(auto_db=auto_db.auto_db,
                            all_cars_table=auto_db.all_cars_table,
                            alimentari=auto_db.alimentari,
                            id_users=current_user.id,
                            id_car=session['id_car'])
        app_masina.delete_row(index)
        return redirect(url_for('consumption_details', table_name=table_name))
    except:
        myerr = 'a_{}_a definition {}\n{}'.format(current_user.id, sys._getframe().f_code.co_name,
                                                  str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/details/<table_name>', methods=['GET', 'POST'])
def consumption_details(table_name):
    try:
        app_masina = Masina(auto_db=auto_db.auto_db,
                            all_cars_table=auto_db.all_cars_table,
                            alimentari=auto_db.alimentari,
                            id_users=current_user.id,
                            id_car=session['id_car'])
        if request.method == 'GET':
            dataFrom, dataBis = app_masina.default_interval
            alim_type = None
            alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
            return render_template('car_detailed.html',
                                   table_name=table_name,
                                   id_car=session['id_car'],
                                   userDetails=alimentari,
                                   types_of_costs=app_masina.types_of_costs,
                                   dataFrom=dataFrom.date(),
                                   dataBis=dataBis.date(),
                                   )
        elif request.method == 'POST':
            # if 'export_detail_table_as_csv' in request.form:
            if 'export_detail_table_as_sql' in request.form:
                tables = {'all_cars': 'user_id',
                          table_name: 'id_users'}
                output_sql_file = current_user.export_car_sql(tables)
                # # return text_to_write
                # pth_to_sql = os.path.join(report_dir, 'aaa.sql')
                # # if os.path.exists(pth_to_sql):
                # #     os.remove(pth_to_sql)
                # # with open(pth_to_sql, 'w') as file:
                # #     for row in text_to_write:
                # #         file.write(row)
                return output_sql_file
            elif 'upload_file' in request.form:
                file = request.files['myfile']
                filename = secure_filename(file.filename)
                fl = os.path.join(rappmysql.report_dir, filename)
                file.save(fl)
                with open(fl) as f:
                    file_content = f.read()
                    print(file_content)
                auto_db.auto_db.run_sql_file(fl)
                return file_content

                # app_masina = Masina(conf.credentials, table_name=table_name.lower())
            elif "filter" in request.form:
                month = request.form['month']
                year = int(request.form['year'])
                alim_type = request.form['type']
                if alim_type == 'all':
                    alim_type = None
                dataFrom = request.form['dataFrom']
                dataBis = request.form['dataBis']

                if month != 'interval':
                    dataFrom, dataBis = app_masina.get_monthly_interval(month, year)
                elif month == 'interval' and (dataFrom == '' or dataBis == ''):
                    dataFrom, dataBis = app_masina.default_interval
                else:
                    try:
                        dataFrom = datetime.strptime(dataFrom, "%Y-%m-%d")
                        dataBis = datetime.strptime(dataBis, "%Y-%m-%d")
                    except:
                        print(traceback.format_exc())
                alimentari = app_masina.get_alimentari_for_interval_type(dataFrom, dataBis, alim_type)
                return render_template('car_detailed.html',
                                       table_name=table_name,
                                       userDetails=alimentari,
                                       types_of_costs=app_masina.types_of_costs,
                                       dataFrom=dataFrom.date(),
                                       dataBis=dataBis.date(),
                                       )
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html', myerr=myerr)


@app.route('/add_masina', methods=['GET', 'POST'])
def add_masina():
    try:
        if current_user.auto_app is None:
            current_user.auto_app = AutoApp(
                auto_db=auto_db.auto_db,
                all_cars_table=auto_db.all_cars_table,
                alimentari=auto_db.alimentari,
                user_id=current_user.id)

        if request.method == 'POST':
            if 'create_user_car' in request.form:
                brand = request.form['brand']
                model = request.form['model']
                car_type = request.form['car_type']
                current_user.auto_app.add_car(brand, model, car_type)
            elif 'import_car_sql' in request.form:
                file = request.files['myfile']
                filename = secure_filename(file.filename)
                fl = os.path.join(rappmysql.report_dir, filename)
                file.save(fl)
                # with open(fl) as f:
                #     file_content = f.read()
                #     print(file_content)
                current_user.run_sql_query(fl)
                os.remove(fl)
                return redirect(url_for('index'))
            elif 'import_car_with_files' in request.form:
                file = request.files['zip_file']
                filename = secure_filename(file.filename)
                fl = os.path.join(rappmysql.report_dir, filename)
                file.save(fl)
                # with open(fl) as f:
                #     file_content = f.read()
                #     print(file_content)
                current_user.import_car_with_files(fl, import_files=True)
                os.remove(fl)
                return redirect(url_for('index'))
            elif 'import_car_without_files' in request.form:
                file = request.files['zip_file']
                filename = secure_filename(file.filename)
                fl = os.path.join(rappmysql.report_dir, filename)
                file.save(fl)
                # with open(fl) as f:
                #     file_content = f.read()
                #     print(file_content)
                current_user.import_car_with_files(fl)
                os.remove(fl)
                return redirect(url_for('index'))
            return redirect(url_for('index'))
        return render_template('addcar.html')
    except:
        myerr = 'a_{}_a definition {}\n{}'.format(current_user.id, sys._getframe().f_code.co_name,
                                                  str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/upload_file/<table_name>/<row_id>', methods=['GET', 'POST'])
def upload_file_to_row(table_name, row_id):
    # print('Module: {}, Def: {}, Caller: {}'.format(__name__, sys._getframe().f_code.co_name,
    #                                                sys._getframe().f_back.f_code.co_name))
    try:
        app_masina = Masina(auto_db=auto_db.auto_db,
                            all_cars_table=auto_db.all_cars_table,
                            alimentari=auto_db.alimentari,
                            id_users=current_user.id,
                            id_car=session['id_car'])

        file = request.files['myfile']
        filename = secure_filename(file.filename)
        fl = os.path.join(rappmysql.report_dir, filename)
        file.save(fl)
        app_masina.upload_file(fl, row_id)
        return 'done'
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html', myerr=myerr)


# @app.before_request
# def before_request():
#     print('Module: {}, Def: {}, Caller: {}'.format(__name__,
#                                                    sys._getframe().f_code.co_name,
#                                                   sys._getframe().f_back.f_code.co_name))
#
#     if current_user.is_authenticated and not hasattr(current_user, 'auto_app'):
#         current_user.auto_app = AutoApp(
#             auto_db=auto_db.auto_db,
#             all_cars_table=auto_db.all_cars_table,
#             alimentari=auto_db.alimentari,
#             user_id=current_user.id)


@app.route('/delete_masina/<id_car>/<table_name>', methods=['GET', 'POST'])
def delete_masina(id_car, table_name):
    try:
        if current_user.auto_app is None:
            current_user.auto_app = AutoApp(
                auto_db=auto_db.auto_db,
                all_cars_table=auto_db.all_cars_table,
                alimentari=auto_db.alimentari,
                user_id=current_user.id)
        current_user.auto_app.delete_auto(table_name)
        return redirect(url_for('index'))
    except:
        myerr = 'a_{}_a definition {}\n{}'.format(current_user.id, sys._getframe().f_code.co_name,
                                                  str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


########################################################################
@app.route('/aeroclub', methods=['GET', 'POST'])
def aeroclub():
    try:
        app_aeroclub = AeroclubSQL(rappmysql.ini_aeroclub)
        print('app_masina.total_unpaid_flights', app_aeroclub.total_unpaid_flights)
        if request.method == 'POST':
            return redirect(url_for('index'))
        return render_template('aeroclub.html',
                               total_unpaid_flights=app_aeroclub.total_unpaid_flights,
                               flight_time_this_year=app_aeroclub.flight_time_this_year,
                               total_flight_time=app_aeroclub.total_flight_time
                               )
    except:
        myerr = 'a_{}_a definition {}\n{}'.format(current_user.id, sys._getframe().f_code.co_name,
                                                  str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


###cheltuieli###################################################################################
@app.route('/cheltuieli', methods=['GET', 'POST'])
@login_required
def cheltuieli():
    try:
        chelt_app = CheltuieliPlanificate(chelt_db=chelt_db.chelt_db,
                                          chelt_plan=chelt_db.chelt_plan,
                                          yearly_plan=chelt_db.yearly_plan)
        dataFrom, dataBis = chelt_plan.default_interval()
        conto = 'all'
        if request.method == 'POST':
            month = request.form['month']
            year = int(request.form['year'])
            conto = request.form['conto']
            dataFrom = request.form['dataFrom']
            dataBis = request.form['dataBis']
            if month != 'interval':
                dataFrom, dataBis = chelt_plan.get_monthly_interval(month, year)
            elif month == 'interval' and (dataFrom == '' or dataBis == ''):
                dataFrom, dataBis = chelt_plan.default_interval()
            else:
                try:
                    dataFrom = datetime.strptime(dataFrom, "%Y-%m-%d")
                    dataBis = datetime.strptime(dataBis, "%Y-%m-%d")
                    print(dataFrom.date(), dataBis.date())
                except:
                    return render_template('cheltuieli.html', tot_val_of_expenses_income=str(traceback.format_exc()))
        try:
            if isinstance(dataFrom, datetime):
                chelt_app.prepareTablePlan(conto, dataFrom.date(), dataBis.date(), True)
            elif isinstance(dataFrom, date):
                chelt_app.prepareTablePlan(conto, dataFrom, dataBis, True)
            return render_template('cheltuieli.html',
                                   expenses=chelt_app.expenses,
                                   tot_no_of_monthly_expenses=chelt_app.tot_no_of_monthly_expenses,
                                   tot_val_of_monthly_expenses=chelt_app.tot_val_of_monthly_expenses,
                                   tot_no_of_irregular_expenses=chelt_app.tot_no_of_irregular_expenses,
                                   tot_val_of_irregular_expenses=chelt_app.tot_val_of_irregular_expenses,
                                   tot_no_of_expenses=chelt_app.tot_no_of_expenses,
                                   tot_val_of_expenses=chelt_app.tot_val_of_expenses,
                                   summary_table=[''],
                                   tot_val_of_income='chelt_app.tot_val_of_income()',
                                   dataFrom=dataFrom,
                                   dataBis=dataBis
                                   )
        except:
            print(traceback.format_exc())
            return render_template('err.html', myerr=str(traceback.format_exc()))
    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


@app.route('/add_cheltuieli', methods=['GET', 'POST'])
@login_required
def add_cheltuieli():
    try:
        chelt_app = CheltuieliPlanificate(rappmysql.ini_chelt)

        if request.method == 'GET':
            return render_template('addchelt.html')
        if request.method == 'POST':
            if "add_one_time_chelt" in request.form:
                name = request.form['name']
                value = request.form['value']
                myconto = request.form['myconto']
                pay_day = request.form['pay_day']
                pay_day = datetime.strptime(pay_day, "%Y-%m-%d")
                chelt_app.add_one_time_transactions(name, value, myconto, pay_day)
                return redirect(url_for('cheltuieli'))

    except:
        myerr = 'definition {}\n{}'.format(sys._getframe().f_code.co_name, str(traceback.format_exc()))
        return render_template('err.html',
                               myerr=myerr)


if __name__ == "__main__":
    app.run()



import subprocess

# pth2ps1skript =
# cmd = ['powershell', r"D:\powershell_skripts\folder_size.ps1"]
cmd = ['powershell', 'tree /f > tree.txt']
p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)

out, err = p.communicate()

if p.returncode != 0:
    print('err', err)
else:
    print('out', out)


import os.path
import pathlib

import mysql.connector as mysql
import re
import codecs
import sys
import traceback
import numpy as np
import subprocess
from datetime import datetime
from rappmysql.mysqlquerys import connect
import rappmysql

compName = os.getenv('COMPUTERNAME')


class DataBase:
    def __init__(self, credentials):
        self.db = mysql.connect(**credentials)
        self.cursor = self.db.cursor()
        self.db_name = credentials['database']

    @property
    def is_connected(self):
        return self.db.is_connected()

    @property
    def dataBaseVersion(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        self.cursor.execute('SELECT version()')
        db_version = self.cursor.fetchone()
        return db_version

    @property
    def allAvailableTablesInDatabase(self):
        """ get all tables in schema"""
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        command = "SHOW TABLES"
        cur.execute(command)
        rows = cur.fetchall()
        tables = []
        for row in rows:
            tabName = row[0]
            tables.append(tabName)
        cur.close()

        return sorted(tables)

    @property
    def allAvailableDatabases(self):
        """ get all tables in schema"""
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        command = 'SHOW DATABASES'
        cur.execute(command)
        rows = cur.fetchall()
        tables = []
        for row in rows:
            tabName = row[0]
            tables.append(tabName)
        cur.close()

        return sorted(tables)

    def checkProcess(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = "SHOW PROCESSLIST"
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        return records

    def killProcess(self, processes):
        for process in processes:
            query = "KILL {}".format(process)
            self.cursor.execute(query)

    def killAllProcess(self, ):
        query = "SHOW PROCESSLIST"
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        for process in records:
            print(process)
            if process[-1] == 'SHOW PROCESSLIST':
                continue
            query = "KILL {}".format(process[0])
            print(query)
            self.cursor.execute(query)

    def run_sql_file(self, file):
        '''
        :param file: QFileDialog.getOpenFileName
        :return:
        '''
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        try:
            error = None
            command = ''
            f = open(file, errors="ignore", encoding="utf-8")
            for line in f.read().splitlines():
                command += line
                if ';' in line:
                    try:
                        print(command)
                        cur = self.db.cursor()
                        cur.execute(command)
                        command = ''
                    except mysql.Error as err:
                        print(20*'+')
                        print(command)
                        print(20 * '+')
                        print('ERROR mysql.Error: ', err.errno)
                        print('ERROR mysql.Error: ', err.msg)
                        error = (err.msg, command)
                        cur.close()
                        break
            f.close()
            self.db.commit()
        except Exception:
            print('ERR: ', traceback.format_exc())
            error = traceback.format_exc()
            cur.close()
        return error

    def createTableFromFile(self, file, newTableName):
        '''
        :param file: QFileDialog.getOpenFileName
        :return:
        '''
        print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        try:
            error = None
            command = ''
            f = open(file)
            for line in f.read().splitlines():
                command += line
                if ';' in line:
                    if re.search("^CREATE TABLE", command) or \
                            re.search("^ALTER TABLE", command) or \
                            re.search("^INSERT INTO", command):
                        match = re.findall(r"\`(.+?)\`", command)
                        tableName = match[0]
                        # newTableName = '{}'.format(tableName)
                        command = command.replace(tableName, newTableName)
                    cur = self.db.cursor()
                    cur.execute(command)
                    command = ''
            f.close()
        except mysql.Error as err:
            print(command, 'ERROR mysql.Error: ', err.msg)
            error = (err.msg, command)
            cur.close()
        except Exception:
            print('ERR: ', traceback.format_exc())
            error = traceback.format_exc()
            cur.close()

        return error

    def createTableList(self, fileList):
        '''
        :param fileList: QFileDialog.getOpenFileNames
        :return: Error if exists, else None
        '''
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        errList = []
        while fileList:
            for i, file in enumerate(fileList):
                error = self.createTableFromFile(file)
                if error:
                    if error[0] == 'Cannot add foreign key constraint' \
                            or re.search("^relation.*does not exist$", error[0]):
                        errList.append(error[1])
                        fileList.pop(i)
                        continue
                    elif 'already exists' in error[0]:
                        fileList.pop(i)
                    else:
                        print('ERR createTablesFromFiles: ', error)
                        fileList.pop(i)
                else:
                    fileList.pop(i)

        probList = []
        for com in errList:
            try:
                print('Retrying... {}'.format(com))
                cur = self.db.cursor()
                cur.execute(com)
                cur.close()
            except Exception:
                print('ERR: ', traceback.format_exc())
                error = (traceback.format_exc(), com)
                probList.append(error)

        if probList:
            print('remaining errs:')
            for i in errList:
                print(i)
        else:
            print('Successfully')

    def drop_table(self, tableName):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = "DROP TABLE IF EXISTS {};".format(tableName)
        cur = self.db.cursor()
        cur.execute(query)
        cur.close()

    def drop_table_list(self, tableList):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        # print(tableList)
        # return
        results = []
        while tableList:
            for i, tab in enumerate(tableList):
                print('######', tab, '#######')
                cur = self.db.cursor()
                # exists = self.checkIfTableExists(tab)
                if tab not in self.allAvailableTablesInDatabase:
                    print('table {} does not exist in database {}'.format(tab, self.db_name))
                    tableList.pop(i)
                    continue
                query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE " \
                        "WHERE table_schema = %s AND referenced_table_name = %s "
                # print(query)
                # print(self.db_name, tab)
                cur = self.db.cursor()
                cur.execute(query, (self.db_name, tab))

                children = []
                for cursor in cur.fetchall():
                    if cursor[0]:
                        children.append(cursor)
                # print('children', children)
                cur.close()
                # return
                if not children:
                    res = self.drop_table(tab)
                    results.append(res)
                    tableList.pop(i)
                else:
                    # if children make sure all of them are included in list
                    for child in children:
                        if child[0] not in tableList:
                            print('if child not in tablelist:', tab, child, children)
                            err = self.drop_table(tab)
                            results.append(err)
                            tableList.pop(i)
                            break
                cur.close()
        return results

    def rename_table(self, tableName, newName):
        # #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = ('RENAME TABLE {} TO {}'.format(tableName, newName))
        cur = self.db.cursor()
        try:
            cur.execute(query)
            cur.close
        except mysql.connector.Error as err:
            print(err.msg)

    def deleteAllDataInTable(self, tableList):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        for tab in tableList:
            query = ('DELETE FROM {}'.format(tab))
            cur.execute(query)
            self.db.commit()
        cur.close()

    def export_database(self, output_file):
        cmd = ['mysqldump', '-u', 'root', '-p', 'cheltuieli_desktop', '>', r"D:\Python\sql_tables\bbb.sql"]
        print('CMD', cmd)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
        out, err = p.communicate()
        if p.returncode != 0:
            print('returncode: ', p.returncode)
            print('Error: ', err)
        else:
            print('Done: {}'.format(cmd))

    def show_create_table(self, tableName):
        query = 'SHOW CREATE TABLE {}'.format(tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        records = records[0]

        cursor.close()
        filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\aa.sql"
        data = records[1]
        FILE = open(filename, "w")
        FILE.writelines(data)
        FILE.close()

    def show_create_table_2(self, tableName):
        print('################')
        query = 'SELECT * FROM {}'.format(tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        records = cursor.fetchall()
        # print('++', query)
        data = ""
        for row in records:
            # print('ÄÄ', row)
            data += "INSERT INTO `" + str(tableName) + "` VALUES("
            first = True
            for field in row:
                if not first:
                    data += ', '
                data += '"' + str(field) + '"'
                first = False

            data += ");\n"
        data += "\n\n"

        cursor.close()
        filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\bb.sql"
        FILE = open(filename, "w")
        FILE.writelines(data)
        FILE.close()

    def export_tables_to_sql(self, tables, output_sql_file):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            cur.execute("SHOW CREATE TABLE `" + str(table) + "`;")
            data += "\n" + str(cur.fetchone()[1]) + ";\n\n"
            cur.execute("SELECT * FROM `" + str(table) + "`;")
            for row in cur.fetchall():
                data += "INSERT INTO `" + str(table) + "` VALUES("
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    data += '"' + str(field) + '"'
                    first = False

                data += ");\n"
            data += "\n\n"
        FILE = open(output_sql_file, "w")
        FILE.writelines(data)
        FILE.close()

    def backup_profile_with_files(self, tables, user_id,  output_dir=None, export_files=True):
        if not output_dir:
            tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
            dir = os.path.dirname(__file__)
            output_dir_name = '{}_{}'.format(tim, '{:09d}'.format(user_id))
            output_dir = os.path.join(dir, r'static\backup_profile', '{:09d}'.format(user_id), output_dir_name)
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
            print('created output_dir', output_dir)
        data = ""
        cur = self.db.cursor()
        for table in tables:
            query = "SHOW CREATE TABLE {}".format(table)
            cur.execute(query)
            show_create_table_query = cur.fetchone()[1]
            if re.search("^CREATE TABLE", show_create_table_query):
                show_create_table_query = show_create_table_query.replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
            # new_create_table_query = []
            # tt = show_create_table_query.split('\n')
            # for r in tt:
            #     if 'file' in r:
            #         continue
            #     if re.search("AUTO_INCREMENT=", r):
            #         match = re.findall(r'AUTO_INCREMENT=(.+?)\ DEFAULT', r)
            #         start_row = match[0]
            #         r = r.replace(start_row, "1")
            #     new_create_table_query.append(r)
            # create_table_query = (('{}\n'*len(new_create_table_query)).format(*new_create_table_query))
            data += "\n {} ;\n\n".format(show_create_table_query)
            # print(data)

            query = 'DESC {}'.format(table)
            cur.execute(query)
            cols_names = cur.fetchall()
            table_has_files = False
            files_cols = []
            export_cols = []
            for col in cols_names:
                if 'file' in col[0]:
                    files_cols.append(col[0])
                    table_has_files = True
                    continue
                export_cols.append(col[0])
            export_cols_query = (('{}, ' * len(export_cols)).format(*export_cols))
            export_cols_query = export_cols_query[:-2]

            query = "SELECT {} FROM {} where {}={};".format(export_cols_query, table, tables[table], user_id)
            cur.execute(query)
            for row in cur.fetchall():
                # print('row****', row, type(row), len(row))
                data += "INSERT IGNORE INTO {} ({}) VALUES(".format(table, export_cols_query)
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    # data += '"' + str(field) + '"'
                    # print(field, type(field))
                    if field is None:
                        data += 'NULL'
                    else:
                        # if table == 'chelt_plan':
                        if isinstance(field, str) and '"' in field:
                            # print(50*'#', '_{}_'.format(field), type(field))
                            field = field.replace('"', "'")
                            # print(50 * '#', '_{}_'.format(field), type(field))
                        data += '"{}"'.format(str(field))
                        # else:
                        #     data += '"{}"'.format(str(field))
                    first = False
                data += ");\n"
            data += "\n\n"
            ###########
            if table_has_files and export_files:
                # print('table', table)
                export_cols_query = (('{}, ' * len(files_cols)).format(*files_cols))
                export_cols_query = 'id, {}'.format(export_cols_query[:-2])
                # print('export_cols_query', export_cols_query)
                query = "SELECT {} FROM {} where {}={};".format(export_cols_query, table, tables[table], user_id)
                cur.execute(query)
                for row in cur.fetchall():
                    # print('ÄÄÄÄ', row)
                    row_id, row_file, row_file_name = row
                    # print('ÄÄÄÄ', row_id, row_file, row_file_name)
                    if not row_file:
                        continue
                    filename = '{}+{}+{}+{}'.format(user_id, table, row_id, row_file_name)
                    filename = os.path.join(output_dir, filename)
                    with open(filename, 'wb') as file:
                        file.write(row_file)

        tim = datetime.strftime(datetime.now(), '%Y_%m_%d__%H_%M_%S')
        output_file = '{}_{}.sql'.format(tim, 'user_name')
        # dir = os.path.dirname(__file__)
        output_sql_file = os.path.join(output_dir, output_file)

        FILE = open(output_sql_file, "w", encoding="utf-8")
        FILE.writelines(data)
        FILE.close()

    def return_sql_text_bkp(self, tables):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            query = "SHOW CREATE TABLE {}".format(table)
            cur.execute(query)
            show_create_table_query = cur.fetchone()[1]
            if re.search("^CREATE TABLE", show_create_table_query):
                show_create_table_query = show_create_table_query.replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")
            data += "\n {} ;\n\n".format(show_create_table_query)
            # print(data)

            query = 'DESC {}'.format(table)
            cur.execute(query)
            cols_names = cur.fetchall()
            table_has_files = False
            files_cols = []
            export_cols = []
            for col in cols_names:
                if 'file' in col[0]:
                    files_cols.append(col[0])
                    table_has_files = True
                    continue
                export_cols.append(col[0])
            export_cols_query = (('{}, ' * len(export_cols)).format(*export_cols))
            export_cols_query = export_cols_query[:-2]
            query = "SELECT {} FROM {} WHERE".format(export_cols_query, table)
            for ccc, vvv in tables[table].items():
                # print(ccc, vvv)
                query += ' {} = {} AND'.format(ccc, vvv)
            query = query[:-4]
            cur.execute(query)
            for row in cur.fetchall():
                # print('row****', row, type(row), len(row))
                data += "INSERT IGNORE INTO {} ({}) VALUES(".format(table, export_cols_query)
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    # data += '"' + str(field) + '"'
                    # print(field, type(field))
                    if field is None:
                        data += 'NULL'
                    else:
                        # if table == 'chelt_plan':
                        if isinstance(field, str) and '"' in field:
                            # print(50*'#', '_{}_'.format(field), type(field))
                            field = field.replace('"', "'")
                            # print(50 * '#', '_{}_'.format(field), type(field))
                        data += '"{}"'.format(str(field))
                        # else:
                        #     data += '"{}"'.format(str(field))
                    first = False
                data += ");\n"
            data += "\n\n"
        return data

    def return_sql_text(self, tables, export_files=False, export_all_users=False):
        data = ""
        cur = self.db.cursor()
        for table in tables:
            query = "SHOW CREATE TABLE {}".format(table)
            cur.execute(query)
            show_create_table_query = cur.fetchone()[1]
            if re.search("^CREATE TABLE", show_create_table_query):
                show_create_table_query = show_create_table_query.replace("CREATE TABLE", "CREATE TABLE IF NOT EXISTS")

            data += "\n {} ;\n\n".format(show_create_table_query)
            # print(data)

            query = 'DESC {}'.format(table)
            cur.execute(query)
            cols_names = cur.fetchall()
            table_has_files = False
            files_cols = []
            export_cols = []
            for col in cols_names:
                if 'file' in col[0]:
                    files_cols.append(col[0])
                    table_has_files = True
                    continue
                export_cols.append(col[0])
            export_cols_query = (('{}, ' * len(export_cols)).format(*export_cols))
            export_cols_query = export_cols_query[:-2]
            if export_all_users:
                query = "SELECT {} FROM {} ".format(export_cols_query, table)
            else:
                query = "SELECT {} FROM {} WHERE".format(export_cols_query, table)
                for ccc, vvv in tables[table].items():
                    query += ' {} = {} AND'.format(ccc, vvv)
                query = query[:-4]
            # print('*-query', query)
            cur.execute(query)
            for row in cur.fetchall():
                # print('row****', row, type(row), len(row))
                data += "INSERT IGNORE INTO {} ({}) VALUES(".format(table, export_cols_query)
                first = True
                for field in row:
                    if not first:
                        data += ', '
                    # data += '"' + str(field) + '"'
                    # print(field, type(field))
                    if field is None:
                        data += 'NULL'
                    else:
                        # if table == 'chelt_plan':
                        if isinstance(field, str) and '"' in field:
                            # print(50*'#', '_{}_'.format(field), type(field))
                            field = field.replace('"', "'")
                            # print(50 * '#', '_{}_'.format(field), type(field))
                        data += '"{}"'.format(str(field))
                        # else:
                        #     data += '"{}"'.format(str(field))
                    first = False
                data += ");\n"
            data += "\n\n"
            if table_has_files and export_files:
                export_cols_query = (('{}, ' * len(files_cols)).format(*files_cols))
                export_cols_query = 'id, {}'.format(export_cols_query[:-2])

                query = "SELECT {} FROM {} WHERE".format(export_cols_query, table)
                for ccc, vvv in tables[table].items():
                    query += ' {} = {} AND'.format(ccc, vvv)
                query = query[:-4]
                cur.execute(query)
                for row in cur.fetchall():
                    row_id, row_file, row_file_name = row
                    if not row_file:
                        continue
                    filename = '{}+{}'.format(row_id, row_file_name)
                    filename = os.path.join(export_files, filename)
                    with open(filename, 'wb') as file:
                        file.write(row_file)

        return data

    def createAliasTempTable(self, tableName, colDict, origTab):
        cur = self.db.cursor()
        selectQuery = ''
        for colName, alias in colDict.items():
            selectQuery += '{} {}, '.format(colName, alias)

        query = "CREATE TEMPORARY TABLE {} " \
                "SELECT {}" \
                " FROM {};".format(tableName, selectQuery[:-2], origTab)
        # print(query)
        cur.execute(query)
        query = ("DESC {}".format(tableName))
        cur.execute(query)
        tableHead = []
        for i in cur.fetchall():
            tableHead.append(i[0])

        query = ('SELECT * FROM {}'.format(tableName))
        cur.execute(query)
        table = cur.fetchall()

        cur.close()

        return tableHead, table

    def descAliasTempTable(self, tableName, colDict, origTab):
        cur = self.db.cursor()
        selectQuery = ''
        for colName, alias in colDict.items():
            selectQuery += '{} {}, '.format(colName, alias)

        query = "CREATE TEMPORARY TABLE {} " \
                "SELECT {}" \
                " FROM {};".format(tableName, selectQuery[:-2], origTab)
        # print(query)
        cur.execute(query)
        query = ("DESC {}".format(tableName))
        cur.execute(query)
        tableHead = []
        for i in cur.fetchall():
            tableHead.append(i[0])

        query = ('SELECT * FROM {}'.format(tableName))
        cur.execute(query)
        table = cur.fetchall()

        cur.close()

        return tableHead, table


class Table(DataBase):
    def __init__(self, credentials, tableName):
        super().__init__(credentials)
        self.tableName = tableName

    @property
    def noOfRows(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = 'SELECT COUNT(*) FROM {}'.format(self.tableName)
        cursor = self.db.cursor()
        cursor.execute(query)
        noOfRows = cursor.fetchone()[0]
        # rowNo = cursor.rowcount
        cursor.close()
        return noOfRows

    def lastRowId(self):
        query = ('SELECT id FROM {} ORDER BY id DESC LIMIT 1'.format(self.tableName))
        cur = self.db.cursor()
        cur.execute(query)
        lastId = cur.fetchonel()
        if lastId is None:
            return 0
        return lastId[0]

    @property
    def columnsNames(self):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cursor = self.db.cursor()
        query = 'DESC {}'.format(self.tableName)
        cursor.execute(query)
        res = cursor.fetchall()
        cols = []
        for col in res:
            cols.append(col[0])
        cursor.close()
        return cols

    @property
    def columnsDetProperties(self):
        query = 'DESC {}'.format(self.tableName)
        colNames = ['Field', 'Type', 'Null', 'Key', 'Default', 'Extra']
        cursor = self.db.cursor()
        cursor.execute(query)
        res = cursor.fetchall()
        cols = {}
        for col in res:
            colName, colType, null, key, default, extra = col
            if isinstance(colType, bytes):
                colType = str(colType.decode("utf-8"))
            cols[colName] = [colType, null, key, default, extra]
        cursor.close()
        return cols

    @property
    def columnsProperties(self):
        #print('Module: {}, Class: {}, Def: {}, Caller: {}'.format(__name__, __class__, sys._getframe().f_code.co_name, sys._getframe().f_back.f_code.co_name))
        cursor = self.db.cursor()
        query = ("SELECT table_name, column_name, data_type FROM INFORMATION_SCHEMA.COLUMNS WHERE table_name = '{}'").format(self.tableName)
        cursor.execute(query)
        res = cursor.fetchall()
        cols = {}
        for col in res:
            table_name, col_name, data_type = col
            cols[col_name] = data_type
        cursor.close()
        return cols

    @property
    def children_tables(self):
        query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE " \
                "WHERE table_schema = %s AND referenced_table_name = %s "
        # print(query)
        # print(self.db_name, self.tableName)
        cur = self.db.cursor()
        cur.execute(query, (self.db_name, self.tableName))
        # query = "SELECT table_name FROM information_schema.KEY_COLUMN_USAGE " \
        #         "WHERE table_schema = {} AND referenced_table_name = {} ".format(self.db_name, self.tableName)
        # print(query)
        # # print(self.db_name, self.tableName)
        # cur = self.db.cursor()
        # cur.execute(query)
        children = []
        for cursor in cur.fetchall():
            # print(cursor)
            if cursor[0]:
                children.append(cursor[0])
        return children

    def rename_column(self, tableName, old_col_name, new_col_name):
        # #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        query = ('ALTER TABLE {} RENAME COLUMN {} TO {}'.format(tableName, old_col_name, new_col_name))
        cur = self.db.cursor()
        try:
            cur.execute(query)
            cur.close
        except mysql.connector.Error as err:
            print(err.msg)

    def deleteRow(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = %s '.format(self.tableName, colName)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def delete_multiple_rows(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = {} '.format(self.tableName, colName, value)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def delete_multiple_rows_multiple_conditions(self, condition):
        colName, value = condition
        query = 'DELETE FROM {} WHERE {} = {} '.format(self.tableName, colName, value)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query, value)
        self.db.commit()
        cursor.close()

    def convertToBinaryData(self, filename):
        # Convert digital data to binary format
        with open(filename, 'rb') as file:
            binaryData = file.read()
        return binaryData

    # def convertToRightType(self, err_msg):
    #     if 'Incorrect decimal value:' in err_msg:

    def addNewRow(self, columns, values):
        # print(len(columns), len(values))
        strCols = (('{}, ' * len(columns)).format(*columns))
        strCols = '({})'.format(strCols[:-2])
        strVals = ('%s,'*len(columns))
        strVals = '({})'.format(strVals[:-1])

        query = "INSERT INTO {} {} VALUES {}".format(self.tableName, strCols, strVals)
        if isinstance(values, int):
            values = (values, )
        elif isinstance(values, str):
            values = (values,)
        elif isinstance(values, tuple):
            # print('values', values)
            new_vals = []
            for v in values:
                if isinstance(v, str):
                    if os.path.isfile(v):
                        v = self.convertToBinaryData(v)
                new_vals.append(v)
            values = tuple(new_vals)

        cursor = self.db.cursor()
        try:
            print(query)
            cursor.execute(query, values)
        except:
            print('nu merge:', traceback.format_exc())
            print()
            print('query: ', query)
            print('values: ', values)
            for i in range(len(columns)):
                print(columns[i], values[i], type(columns[i]), type(values[i]))
            raise RuntimeError
        # try:
        #     cursor.execute(query, values)
        # except mysql.errors.DatabaseError as err:
        #     print(err)
        #     print(err.msg)
        #     print('ÄÄÄÄÄÄÄÄÄÄÄ')
        #     self.convertToRightType(err.msg)
        self.db.commit()
        cursor.close()

        return cursor.lastrowid

    def addNewRowWithFile(self, columns, values):
        # print(len(columns), len(values))
        strCols = (('{}, ' * len(columns)).format(*columns))
        strCols = '({})'.format(strCols[:-2])
        strVals = ('%s,'*len(columns))
        strVals = '({})'.format(strVals[:-1])

        query = "INSERT INTO {} {} VALUES {}".format(self.tableName, strCols, strVals)
        #######
        print('***',query)
        # for i in range(len(columns)):
        #     print(columns[i], values[i])
        #######
        if isinstance(values, int):
            values = (values, )
        elif isinstance(values, str):
            values = (values,)
        elif isinstance(values, tuple):
            # print('values', values)
            new_vals = []
            for v in values:
                if isinstance(v, pathlib.WindowsPath):
                    print('aaaaa', v)
                    if v.is_file():
                        v = self.convertToBinaryData(v)
                new_vals.append(v)
            values = tuple(new_vals)
        # print('-->', values)
        try:
            cursor = self.db.cursor()
            cursor.execute(query, values)
            self.db.commit()
            cursor.close()
            print('executed')
            return cursor.lastrowid
        except mysql.Error as err:
            if err.errno == 2013 and err.msg == 'Lost connection to MySQL server during query':
                print('BINOGOOO')
                print('ERROR mysql.Error: ', err.errno)
                print('ERROR mysql.Error: ', err.msg)
                cursor.execute('set max_allowed_packet=67108864')
                self.db.commit()
                cursor.execute(query, values)
                self.db.commit()
                cursor.close()
                print('DONEEEEE')
            else:
                print('Alta eroare: ', err.errno, err.msg)

    def insertColumns(self, column_name, column_definition, afterCol):
        if afterCol == 'FIRST':
            query = 'ALTER TABLE {} ADD COLUMN {} {} FIRST'.format(self.tableName, column_name, column_definition)
        else:
            query = 'ALTER TABLE {} ADD COLUMN {} {} AFTER {}'.format(self.tableName, column_name, column_definition, afterCol)

        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def returnAllRecordsFromTable(self):
        cur = self.db.cursor()
        query = ('SELECT * FROM {}'.format(self.tableName))
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnAllRecordsFromTableExceptBlob(self):
        cur = self.db.cursor()
        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {}'.format(self.tableName)
        print(query)
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnLastRecords(self, column, noOfRows2Return):
        #print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        cur = self.db.cursor()
        query = ('SELECT * FROM {} ORDER BY {} DESC LIMIT %s'.format(self.tableName, column))
        cur.execute(query, (noOfRows2Return,))
        rows = cur.fetchall()
        cur.close()
        return rows

    def filterRows(self, matches, order_by=None):
        # print('Module: {}, Class: {}, Def: {}'.format(__name__, __class__, sys._getframe().f_code.co_name))
        filterText = ''
        for match in matches:
            search_col, search_key = match
            if isinstance(search_key, tuple):
                min, max = search_key
                new = "{} >= '{}' AND {} <= '{}' AND ".format (search_col, min, search_col, max)
                filterText += new
            elif isinstance(search_key, list):
                new = "{} in {} AND ".format(search_col, tuple(search_key))
                filterText += new
            elif search_key == 'None' or search_key is None:
                new = "{} IS NULL AND ".format(search_col, search_key)
                filterText += new
            else:
                new = "{} = '{}' AND ".format(search_col, search_key)
                filterText += new

        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])

        # print(query)
        # query = "SELECT * FROM {} WHERE ".format(self.tableName) + filterText[:-4]
        if order_by:
            col, order = order_by
            txt = 'ORDER BY {} {}'.format(col, order)
            query += txt
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        return records

    def returnRowsWhere(self, matches):
        has_longblob_col = False
        cols_without_longblob = ''
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                has_longblob_col = True
            else:
                cols_without_longblob += '{}, '.format(col_name)
        cols_without_longblob = cols_without_longblob[:-2]

        # print('cols_without_longblob', cols_without_longblob)
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT * FROM {} WHERE {} = '{}'".format(self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT * FROM {} WHERE {} IN '{}'".format(self.tableName, searchCol, searchKey)
            if searchKey is None:
                query = "SELECT * FROM {} WHERE {} IS NULL".format(self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    new = '{} IS NULL AND '.format(searchCol)
                else:
                    new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT * FROM {} WHERE ".format(self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        if has_longblob_col:
            query = query.replace('*', cols_without_longblob)
        print('query', query)
        # return
        cursor = self.db.cursor()

        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def returnRowsLike(self, column, keyWord):
        query = "SELECT * FROM {} WHERE {} LIKE '%{}%'".format(self.tableName,
                                                               column, keyWord)
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        return records

    def returnRowsYoungerThan(self, column, timeP):
        query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, column, column)
        cur = self.db.cursor()
        cur.execute(query, (timeP, ))
        records = cur.fetchall()
        return records

    def returnRowsInInterval(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} > %s OR {} < %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def returnRowsWhereValueIsInIntervalAND(self, colWhere, keyWord, startColumn, valueInInterval, endColumn):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} = %s AND %s > {} AND {} > %s".format(self.tableName, colWhere, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (keyWord, valueInInterval, valueInInterval))
        records = cur.fetchall()
        return records

    def returnCellWhereValueIsInIntervalAND(self, col2return,  colWhere, keyWord, startColumn, valueInInterval, endColumn):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT {} FROM {} WHERE {} = %s AND %s > {} AND ({} > %s OR {} IS NULL)".format(col2return, self.tableName, colWhere, startColumn, endColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (keyWord, valueInInterval, valueInInterval))
        records = cur.fetchall()
        return records

    def returnRowsQuery(self, matches, order_by=None):
        filterText = ''
        for match in matches:
            search_col, sign, search_value = match
            if sign == 'LIKE':
                new = "{} {} '%{}%' AND ".format(search_col, sign, search_value)
                filterText += new
            elif search_value == None:
                new = "{} {} NULL AND ".format(search_col, sign, search_value)
                filterText += new
            else:
                new = "{} {} '{}' AND ".format(search_col, sign, search_value)
                filterText += new

        query = 'SELECT '
        for col_name, prop in self.columnsDetProperties.items():
            # print(col_name, prop)
            if prop[0] == 'longblob':
                continue
            query += '{}, '.format(col_name)
        query = query[:-2]
        query += ' FROM {} WHERE {} '.format(self.tableName, filterText[:-4])
        # query = 'SELECT * FROM {} WHERE {} '.format(self.tableName, filterText[:-4])
        if order_by:
            col, order = order_by
            txt = 'ORDER BY {} {}'.format(col, order)
            query += txt
        # print('__query__', query)
        cur = self.db.cursor()
        cur.execute(query)
        records = cur.fetchall()
        cur.close()
        return records

    def returnRowsOfYear(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} >= %s AND {} <= %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def returnRowsOutsideInterval(self, startColumn, startTime, endColumn, endTime):
        # query = "SELECT * FROM {} WHERE {} > %s ORDER BY {} DESC".format(self.tableName, startColumn, startColumn)
        query = "SELECT * FROM {} WHERE {} < %s AND {} > %s".format(self.tableName, startColumn, endColumn)
        # print(query)
        cur = self.db.cursor()
        cur.execute(query, (startTime, endTime))
        records = cur.fetchall()
        return records

    def get_column_type(self, column):
        colProps = self.columnsProperties[column]
        # print('µµµµµµµµµ', colProps)
        colType = colProps[0]
        return colType

    def modify2AutoIncrement(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {} AUTO_INCREMENT;'.format(self.tableName, column, colType)
        print(query)
        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def modifyType(self, column, colType):
        query = 'ALTER TABLE {} MODIFY {} {};'.format(self.tableName, column, colType)
        cursor = self.db.cursor()
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def changeCellContent(self, column2Modify, val2Moify, refColumn, refValue):
        cursor = self.db.cursor()
        try:
            if isinstance(refValue, tuple):
                query = "UPDATE {} SET {} = {} WHERE {} IN {}".format(self.tableName, column2Modify, val2Moify, refColumn, refValue)
                cursor.execute(query)
                self.db.commit()
                cursor.close()
                return
            else:
                query = "UPDATE {} SET {} = %s WHERE {} = %s".format(self.tableName, column2Modify, refColumn)
            print(query)
            print('val2Moify', val2Moify)

            if isinstance(val2Moify, str):
                if os.path.isfile(val2Moify):
                    print('aaaaa', val2Moify, type(val2Moify))
                    val2Moify = self.convertToBinaryData(val2Moify)
                    # print('aaaaa', val2Moify, type(val2Moify))
            vals = (val2Moify, int(refValue))
            # print(vals)
            cursor.execute(query, vals)
            self.db.commit()
            cursor.close()
        except mysql.Error as err:
            print('++', err)
            print('query', query)
            print('vals', vals)
            print('ERROR mysql.Error: ', err.msg)
            # error = err.msg
            # cur.close()

    def dropColumn(self, column2Del):
        query = "ALTER TABLE {} DROP COLUMN %s;".format(self.tableName)
        query = "ALTER TABLE {} DROP COLUMN {};".format(self.tableName, column2Del)
        print(query)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        self.db.commit()
        cursor.close()

    def executeQuery(self, query):
        print(sys._getframe().f_code.co_name)
        # print(file)
        cursor = self.db.cursor()
        if isinstance(query, str):
            commands = query.split(';')
        for command in commands:
            print('executing command: ', command)
            cursor.execute(command)

    def convertDatumFormat4SQL(self, datum):
        # print(sys._getframe().f_code.co_name)
        # newDate = datetime.strptime(datum, '%d.%m.%y')
        for fmt in ('%Y-%m-%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
            try:
                newDate = datetime.strptime(datum, fmt)
                return newDate.date()
            except ValueError:
                pass
        raise ValueError('no valid date format found: {}'.format(datum))

    def convertDateTimeFormat4SQL(self, datum):
        # print(sys._getframe().f_code.co_name)
        # newDate = datetime.strptime(datum, '%d.%m.%y') 2024/09/21 11:10
        for fmt in ('%Y/%m/%d %H:%M', '%Y-%m-%d %H:%M'):
            try:
                newDate = datetime.strptime(datum, fmt)
                return newDate
            except ValueError:
                pass
        raise ValueError('no valid date format found: {}'.format(datum))

    def convertTimeFormat4SQL(self, time):
        # print(sys._getframe().f_code.co_name)
        # newDate = datetime.strptime(datum, '%d.%m.%y')
        for fmt in ('%H:%M', '%H:%M:%S'):
            try:
                newDate = datetime.strptime(time, fmt)
                return newDate.time()
            except ValueError:
                pass
        raise ValueError('no valid date format found')

    def returnColumn(self, col):
        query = 'SELECT {} FROM {}'.format(col, self.tableName)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i[0])
        return values

    def returnColumns(self, cols):
        strTableHead = ''
        for col in cols:
            strTableHead += '{}, '.format(col)
        strTableHead = strTableHead[:-2]

        query = 'SELECT {} FROM {}'.format(strTableHead, self.tableName)
        cursor = self.db.cursor()
        # vals = (column2Del, )
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def returnCellsWhere(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(col, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN {}".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        # print('*****records', records, type(records))
        cursor.close()
        values = []
        # colType = self.get_column_type(col)
        for i in records:
            # print('ßßßßßßß', colType)
            # print('ßßßßßßß', i[0])
            # if colType == 'longblob':
            #     values.append(i[0])
            #     # continue
            # # elif colType == 'json':
            # #     values.append(json.loads(i[0]))
            # else:
            values.append(i[0])
        return values

    def returnCellsWhereDiffrent(self, col, matches):
        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} != '{}'".format(col, self.tableName, searchCol, searchKey)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                new = '{} != "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(col, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        # print(records)
        cursor.close()
        values = []
        colType = self.get_column_type(col)
        for i in records:
            if colType == 'json':
                values.append(json.loads(i[0]))
            else:
                values.append(i[0])
        return values

    def returnColsWhere(self, cols, matches):
        relCols = ''
        for col in cols:
            relCols += '{}, '.format(col)
        relCols = relCols[:-2]

        if isinstance(matches, tuple):
            searchCol, searchKey = matches
            if isinstance(searchKey, str) or isinstance(searchKey, int):
                query = "SELECT {} FROM {} WHERE {} = '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if isinstance(searchKey, tuple):
                query = "SELECT {} FROM {} WHERE {} IN '{}'".format(relCols, self.tableName, searchCol, searchKey)
            if searchKey is None:
                query = "SELECT {} FROM {} WHERE {} IS NULL".format(relCols, self.tableName, searchCol)
        elif isinstance(matches, list):
            text = ''
            for i in matches:
                searchCol, searchKey = i
                if searchKey is None:
                    new = '{} IS NULL AND '.format(searchCol)
                else:
                    new = '{} = "{}" AND '.format(searchCol, searchKey)
                text += new
            query = "SELECT {} FROM {} WHERE ".format(relCols, self.tableName) + text[:-4]
        else:
            raise TypeError('{} must be tuple or list of tuples'.format(matches))

        cursor = self.db.cursor()
        # print('query', query)
        cursor.execute(query)
        records = cursor.fetchall()
        cursor.close()
        values = []
        for i in records:
            values.append(i)
        return values

    def write_file(self, data, filename):
        # Convert binary data to proper format and write it on Hard Disk
        with open(filename, 'wb') as file:
            file.write(data)

    # def show_create_table_2(self, tableName):
    #     print('################')
    #     query = 'SELECT * FROM {}'.format(self.tableName)
    #     cursor = self.db.cursor()
    #     cursor.execute(query)
    #     records = cursor.fetchall()
    #     # print('++', query)
    #     data = ""
    #     for row in records:
    #         # print('ÄÄ', row)
    #         data += "INSERT INTO `" + str(self.tableName) + "` VALUES("
    #         first = True
    #         for field in row:
    #             if not first:
    #                 data += ', '
    #             data += '"' + str(field) + '"'
    #             first = False
    #
    #         data += ");\n"
    #     data += "\n\n"
    #
    #     values = []
    #     # for i in data:
    #     #     print(i)
    #     cursor.close()
    #     #     # values.append(i[0])
    #     # # return values
    #     # filename = str(os.getenv("HOME")) + "/backup_" + now.strftime("%Y-%m-%d_%H:%M") + ".sql"
    #     filename = r"C:\_Development\Diverse\pypi\radu\masina\src\masina\bb.sql"
    #     # data = records[1]
    #     FILE = open(filename, "w")
    #     FILE.writelines(data)
    #     FILE.close()

    def compare_sql_file_to_sql_table(self, sql_file):
        f = open(sql_file)
        same = True
        diffrences = {}
        for line in f.read().splitlines():
            spl_row = line.split()
            if spl_row and '`' in spl_row[0]:
                sql_file_col = spl_row[0].strip('`')
                if sql_file_col in self.columnsDetProperties.keys():
                    col_type_file = spl_row[1]
                    col_type_sql = self.columnsDetProperties[sql_file_col][0]
                    if col_type_file != col_type_sql:
                        txt = 'Column {} has a different format in file than in sql table'.format(sql_file_col)
                        # print(txt)
                        same = False
                        diffrences[sql_file_col] = txt
                        # raise RuntimeError(txt)
                else:
                    print('sql_file_col', sql_file_col)
                    print('self.columnsDetProperties.keys()', self.columnsDetProperties.keys())
                    raise RuntimeError('Column {} missing in sql table file {}'.format(sql_file_col, sql_file))
        f.close()
        if not same:
            return diffrences


def convertDatumFormat4SQL(datum):
    # print(sys._getframe().f_code.co_name)
    # newDate = datetime.strptime(datum, '%d.%m.%y')
    for fmt in ('%Y%m%d','%Y-%m-%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
        try:
            newDate = datetime.strptime(datum, fmt)
            return newDate.date()
        except ValueError:
            pass
    raise ValueError('no valid date format found: {}'.format(datum))


def convertDateTimeFormat4SQL(datum):
    # print(sys._getframe().f_code.co_name)
    # newDate = datetime.strptime(datum, '%d.%m.%y') 2024/09/21 11:10
    for fmt in ('%Y/%m/%d %H:%M', '%Y-%m-%d %H:%M', '%Y%m%d_%H%M%S'):
        try:
            newDate = datetime.strptime(datum, fmt)
            return newDate
        except ValueError:
            pass
    raise ValueError('no valid date format found: {}'.format(datum))


if __name__ == '__main__':


    # if compName == 'DESKTOP-5HHINGF':
    #     ini_file = r"D:\Python\MySQL\cheltuieli.ini"
    # else:
    #     ini_file = r"C:\_Development\Diverse\pypi\cfgm.ini"
    #     # ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"

    ini_file = rappmysql.ini_chelt

    conf = connect.Config(ini_file)
    # print(conf.credentials)

    db = DataBase(conf.credentials)
    print(db.is_connected)

    # tables_dict = {'users': 'id',
    #                'all_cars': 'user_id',
    #                'masina': 'id_users'}
    # tables_dict = {'users': 'id',
    #                'banca': 'id_users',
    #                'income': 'id_users',
    #                'real_expenses_1': 'id_users',
    #                'chelt_plan': 'id_users',
    #                }

    # tables_dict = {'users': 'id',
    #                'banca': 'id_users',
    #                'chelt_plan': 'id_users',
    #                'knowntrans': 'id_users',
    #                'income': 'id_users',
    #                'deubnk': 'id_users',
    #                'n26': 'id_users',
    #                'sskm': 'id_users',
    #                'plan_vs_real': 'id_users',
    #                'imported_csv': 'id_users'
    #                }

    # output_dir = r'C:\_Development\Diverse\pypi\radu\masina\src\masina\static\backup_profile\000000001\000000001'
    # db.backup_profile_with_files(tables_dict, user_id=1, output_dir=output_dir, export_files=True)

    # db.drop_table_list(tab_list)

    matches = [('type', 'benzina'),
               ('id_users', 1),
               ('id_all_cars', 1)]

    ini_file = rappmysql.ini_masina

    conf = connect.Config(ini_file)
    credentials = conf.credentials
    alimentari = Table(credentials, 'masina')
    rows = alimentari.returnRowsWhere(matches)
    for ii in rows:
        print(ii)


import csv
import os
import sys
import re
import numpy as np
from datetime import datetime
import dateutil.parser as dparser
from mysqlquerys import connect
from mysqlquerys import mysql_rm

np.set_printoptions(linewidth=250)

db_type = 'MySQL'
ini_file = r"D:\Python\MySQL\database.ini"
flightTable = connect.Table(ini_file, 'aeroclub', 'zbor_log')
costsTable = connect.Table(ini_file, 'myfolderstructure', 'aeroclub')


def write2flightTable(inpFile):
    with open(inpFile, 'r', encoding='unicode_escape', newline='') as csvfile:
        linereader = csv.reader(csvfile, delimiter=';', quotechar='"')
        for i, row in enumerate(linereader):
            if i == 0:
                tableHead = [c.strip('"') for c in row]
                # print(tableHead)
                tabHeadDict = {'Lfz.': 'plane',
                               'Datum': 'flight_date',
                               'Start': 'start',
                               'Landung': 'land',
                               'Zeit': 'flight_time',
                               'Startort': 'place_from',
                               'Landeort': 'place_to'
                               }
                continue
            cols = []
            vals = []

            for ir, v in enumerate(row):
                origColName = tableHead[ir]
                # print(origColName, list(tabHeadDict.keys()))
                if origColName in list(tabHeadDict.keys()):
                    # print('BINGO')
                    cols.append(tabHeadDict[origColName])
                    if tabHeadDict[origColName] == 'flight_date':
                        print('before', v)
                        v = flightTable.convertDatumFormat4SQL(v)
                        print('after', v)
                    elif (tabHeadDict[origColName] == 'Start') or (tabHeadDict[origColName] == 'Landung'):
                        v = flightTable.convertTimeFormat4SQL(v)
                    elif tabHeadDict[origColName] == 'Zeit':
                        v = int(v)
                    vals.append(v)

            print('cols', cols)
            print('vals', vals)
            cols.append('price_hour')
            vals.append(85)
            flightTable.add_row(cols, vals)


def getPrice4Time(fromTime, bis):
    match1 = ('flight_date', (fromTime, bis))
    match2 = ('plane', 'D-MENF')
    matches = [match1, match2]

    flights = flightTable.filterRows(matches)
    price = 0
    for row in flights:
        flightTime = row[flightTable.columnsNames.index('flight_time')]
        pricePerHour = row[flightTable.columnsNames.index('price_hour')]
        pricePerFlight = (int(flightTime) * float(pricePerHour))/60
        price += pricePerFlight
    price = round(price, 2)

    return price


def getQuartalDates(quartal, year):
    if quartal == 'Q1':
        quartal = (datetime(year, 1, 1), datetime(year, 3, 31))
    elif quartal == 'Q2':
        quartal = (datetime(year, 4, 1), datetime(year, 6, 30))
    elif quartal == 'Q3':
        quartal = (datetime(year, 7, 1), datetime(year, 9, 30))
    elif quartal == 'Q4':
        quartal = (datetime(year, 10, 1), datetime(year, 12, 31))
    return quartal


def getVideo(path):
    foundDirs = []
    dir = os.listdir(path)
    for d in dir:
        pth = os.path.join(path, d)
        if os.path.isdir(pth):
            try:
                # aa = dparser.parse(d, fuzzy=True)
                # print(aa)
                match = re.search(r'\d{4}.\d{2}.\d{2}', d)
                name = d[match.span()[1]:]
                date = datetime.strptime(match.group(), '%Y.%m.%d').date()
                tup = (date, name, pth)
                foundDirs.append(tup)
            except:
                print('****', d)
                continue

    for row in foundDirs:
        data, name, pth = row
        # print(flightTable.columnsNames.index('flight_date'))
        # print(flightTable.data[:, 3])
        flighttable = np.atleast_2d(flightTable.data)
        indx = np.where(flighttable[:, flightTable.columnsNames.index('flight_date')] == data)
        # print(flighttable[indx])
        for i in flighttable[indx]:
            id = i[0]
            flightTable.changeCellContent('name', str(name), 'id', id)
            flightTable.changeCellContent('path2log', str(pth), 'id', id)


# def write2costsTable(price):
#     costsTable


if __name__ == '__main__':
    # inpFile = r"D:\Documente\Aeroclub\Bad_Endorf\Export.csv"
    inpFile = r"D:\Documente\Radu\Aeroclub\Flight_Log\Export_VEREINSFLIEGER_11.10.2021_17.09.2024.csv"
    write2flightTable(inpFile)
    # quartals = ['Q1', 'Q2', 'Q3', 'Q4']
    # for year in range(2022, 2023):
    #     for q in quartals:
    #         interval = getQuartalDates(q, year)
    #         print(interval)
    # #         price = getPrice4Time(interval[0], interval[1])
    # #         print(year, q, price)
    # # pth = r"E:\Aviatie\Filmulete zbor"
    # # getVideo(pth)


import os
import time
import csv
from datetime import date
from mysqlquerys import mysql_rm
import connect
from datetime import datetime


def insert_file_to_db():
    #https://pynative.com/python-mysql-blob-insert-retrieve-file-image-as-a-blob-in-mysql/
    ini_file = r"D:\Python\MySQL\test_db.ini"
    txt_file = r"D:\Documente\Munca\ALTEN\Arbeitszeugnis.pdf"
    with open(txt_file, 'rb') as file:
        binaryData = file.read()
    conf = connect.Config(ini_file)
    active_table = mysql_rm.Table(conf.credentials, 'testrrrr')
    query = "INSERT INTO testrrrr (aaa, ccc, biodata) VALUES (%s,%s,%s)"
    insert_blob_tuple = (111, 'aaa', binaryData)
    cursor = active_table.db.cursor()
    result = cursor.execute(query, insert_blob_tuple)
    active_table.db.commit()
    print("Image and file inserted successfully as a BLOB into python_employee table", result)
    cursor.close()


def get_blob():
    test_db = r"D:\Python\MySQL\test_db.ini"
    conf = connect.Config(test_db)
    active_table = mysql_rm.Table(conf.credentials, 'testrrrr')
    print(active_table.columnsProperties)

    cursor = active_table.db.cursor()
    # print(active_table.columnsNames)
    file_name = r'D:\Python\MySQL\test.pdf'
    id = 9
    sql_fetch_blob_query = "SELECT * from testrrrr where id = %s"
    cursor.execute(sql_fetch_blob_query, (id,))
    record = cursor.fetchall()
    for row in record:
        data = row[3]

        with open(file_name, 'wb') as file:
            file.write(data)


def filterRows_order_by():
    chelt_db = r"D:\Python\MySQL\cheltuieli_db.ini"
    conf = connect.Config(chelt_db)
    active_table = mysql_rm.Table(conf.credentials, 'alimentari')
    selectedStartDate = date(2021, 1, 1)
    selectedEndDate = date(2023, 12, 31)
    matches = [('data', (selectedStartDate, selectedEndDate)), ('type', 'benzina')]

    order_by = ('data', 'DESC')

    table = active_table.filterRows(matches, order_by)
    for i in table:
        print(i)


def return_cells_where():
    chelt_db = r"D:\Python\MySQL\cheltuieli_db.ini"
    ini_file = r"D:\Python\MySQL\cheltuieli_online\src\rappmysql\static\wdb.ini"
    test_db = r"D:\Python\MySQL\test_db.ini"

    conf = connect.Config(chelt_db)

    active_table = mysql_rm.Table(conf.credentials, 'hyundai_ioniq')
    print(active_table.columnsProperties)

    matches = [('type', 'electric')]
    ids = active_table.returnCellsWhere('id',matches)
    # # money = active_table.returnCellsWhere('id', matches)
    print(ids)
    for i in ids:
    #     # match = [('id', i)]
    #     # val = active_table.returnCellsWhere('type', match)
    #     # print(val[0])
    #     active_table.changeCellContent('type', 'electric', 'id', i)
        xx = ['id', (i, )]
        active_table.deleteRow(xx)
    cols = ['data', 'type', 'brutto', 'amount', 'ppu', 'comment',  'refuel', 'other', 'recharges']
    inpFile = r"D:\Documente\Masina\Hyundai Ioniq\Consum\rechnungen.csv"
    with open(inpFile, 'r', encoding='unicode_escape', newline='') as csvfile:
        linereader = csv.reader(csvfile, delimiter=';', quotechar='|')
        for i, row in enumerate(linereader):
            print(i, row)
            if i == 0:
                continue
            data, comment, amount,	brutto,	refuel,	other, recharges, ppu = row
            try:
                recharges = int(recharges)
            except:
                recharges = 0
            try:
                other = float(other)
            except:
                other = 0
            try:
                ppu = float(ppu)
            except:
                ppu = 0
            vals = (data, 'electric', brutto, amount,  ppu, comment, refuel, other, recharges)
            active_table.addNewRow(cols, vals)

    db = mysql_rm.DataBase(conf.credentials)
    db.export_database(None)


def convertDatumFormat4SQL(datum):
    # print(sys._getframe().f_code.co_name)
    # newDate = datetime.strptime(datum, '%d.%m.%y')
    for fmt in ('%Y-%m-%d','%Y_%m_%d', '%d.%m.%Y', '%d/%m/%Y', '%m/%d/%Y', '%d.%m.%y'):
        try:
            newDate = datetime.strptime(datum, fmt)
            return newDate.date()
        except ValueError:
            pass
    raise ValueError('no valid date format found')



def main():
    script_start_time = time.time()
    selectedStartDate = date(2023, 11, 20)
    selectedEndDate = date(2023, 11, 30)

    # chelt_db = r"D:\Python\MySQL\cheltuieli_db.ini"
    chelt_db = r"D:\Python\MySQL\masina.ini"
    ini_file = r"C:\_Development\Diverse\pypi\cfgmRN.ini"


    conf = connect.Config(ini_file)
    print(conf.credentials)
    print(conf.db_type)
    # print(type(conf.credentials))
    # active_table = mysql_rm.Table(conf.credentials, 'hyundai_ioniq')

    # dir = r'D:\Documente\Masina\Hyundai Ioniq\Consum\IoniqElectroLaden\SWM'
    # all = os.listdir(dir)
    # found = 0
    # for fl in all:
    #     print(fl)
    #     date_str = fl[:10]
    #     # print(date_str)
    #     date_d = convertDatumFormat4SQL(date_str)
    #     # print(date_d, type(date_d))
    #     # print(date_d)
    #     matches = [('type', 'electric'), ('data', date_d), ('comment', 'SWM')]
    #     ids = active_table.returnCellsWhere('id', matches)
    #     # print('....', ids)
    #     if len(ids) == 1:
    #         id_i = ids[0]
    #         inpFile = os.path.join(dir, fl)
    #         print(fl)
    #         v = active_table.convertToBinaryData(inpFile)
    #         active_table.changeCellContent('file', v, 'id', id_i)
    #         active_table.changeCellContent('file_name', fl, 'id', id_i)
    #         # print(fl, date_d, id_i)
    #         found += 1
    #     elif len(ids) == 0:
    #         print('NOT FOUND', fl, date_d)
    #     elif len(ids) > 1:
    #         print('MORE', fl, date_d)
    # print('total_chitante {} total importate {}'.format(len(all), found))


    # inpFile = r"D:\Documente\Masina\Hyundai Ioniq\Consum\IoniqElectroLaden\EnBW\2022-07-05 Rechnung_2022_06_DE_180010096839.pdf"
    # v = active_table.convertToBinaryData(inpFile)
    # active_table.changeCellContent('file', v, 'id', 613)


    # pth = r"D:\Python\MySQL\cheltuieli_online\src\rappmysql\static\sql\auto.sql"
    # active_db = mysql_rm.DataBase(conf.credentials)
    # active_db.createTableFromFile(pth)

    scrip_end_time = time.time()
    duration = scrip_end_time - script_start_time
    duration = time.strftime("%H:%M:%S", time.gmtime(duration))
    print('run time: {}'.format(duration))


if __name__ == '__main__':
    main()
